/* -----------------------
   File Wave.pro
----------------------- */

/*======================
  Geometrical Entities
======================*/

Group {
   // Boundary
   Gama0 = Region[ 1001 ];
   Gama1 = Region[ 1002 ];

   // The Domain
   Omega = Region[ 1005 ];
   
   // The Boundary And The Domain
   AllOmega = Region[{Omega,Gama0,Gama1}];
}

/*=====================
   Some Functions
=====================*/

Function {
   // Initial Position of the membrane
   InitialState[AllOmega] = 10*Exp[-25*((X[]+0.5)*(X[]+0.5)+(Y[]+0.5)*(Y[]+0.5))] - 5*Exp[-50*((X[]+0.3)*(X[]+0.3)+(Y[]+0.5)*(Y[]+0.5))] - 10*Exp[-45*((X[]-0.5)*(X[]-0.5)+(Y[]-0.5)*(Y[]-0.5))];
   // Initial Velocity of the membrane
   InitialVelocity[AllOmega] = -Exp[-25*((X[]+0.5)*(X[]+0.5)+(Y[]-0.6)*(Y[]-0.6))];

   // Control (Up and down oscillations)
   u[] = 5*Sin[2*Pi*$Time];

   // Time Discretization
   t0 = 0.;
   T = 2.;
   dt = 0.01;
   
   // Type of Scheme (See Newmark on the manual)
   beta = 0.25;
   gamma = 0.5;
}

/*===============================
  Constraints and Initial State
===============================*/

Constraint {
   // Initial State
   { Name InitialData; Type Init;
      Case {
         { Region Omega; Value InitialState[]; }
      }
   }
   // Boundary control
   { Name Displacement; Type Assign;
      Case {
         { Region Gama1; Value 1.; TimeFunction u[]; }
      }
   }
   // Boundary Dirichlet Condition (membrane fixed on Gama0)
   { Name Fixed; Type Assign;
      Case {
         { Region Gama0; Value 0.; }
      }
   }
}

/*====================
  Functional Spaces
====================*/

FunctionSpace {
   { Name VhNeu; Type Form0;
      BasisFunction{
      { Name wn; NameOfCoef vn; Function BF_Node;
         Support AllOmega; Entity NodesOf[All];}
      }
      Constraint {
      { NameOfCoef vn; EntityType NodesOf;
         NameOfConstraint Displacement; }
      { NameOfCoef vn; EntityType NodesOf;
         NameOfConstraint InitialData; }
      }
   }
   { Name VhDir; Type Form0;
      BasisFunction{
      { Name wn; NameOfCoef vn; Function BF_Node;
         Support AllOmega; Entity NodesOf[All];}
      }
      Constraint {
      { NameOfCoef vn; EntityType NodesOf;
         NameOfConstraint Displacement; }
      { NameOfCoef vn; EntityType NodesOf;
         NameOfConstraint Fixed; }
      { NameOfCoef vn; EntityType NodesOf;
         NameOfConstraint InitialData; }
      }
   }
}

/*====================
    Jacobian
====================*/

Jacobian {
  { Name JVol ;
    Case {
      { Region All ; Jacobian Vol ; }
    }
  }
  { Name JSur ;
    Case {
      { Region All ; Jacobian Sur ; }
    }
  }
  { Name JLin ;
    Case {
      { Region All ; Jacobian Lin ; }
    }
  }
}

/*======================
   Integral Parameters
======================*/

Integration {
  { Name I1 ;
    Case {
      { Type Gauss ;
        Case {
          { GeoElement Point ; NumberOfPoints  1 ; }
          { GeoElement Line ; NumberOfPoints  4 ; }
          { GeoElement Triangle ; NumberOfPoints  6 ; }
          { GeoElement Quadrangle ; NumberOfPoints 7 ; }
          { GeoElement Tetrahedron ; NumberOfPoints 15 ; }
          { GeoElement Hexahedron ; NumberOfPoints 34 ; }
        }
      }
    }
  }
}

/*======================
  Weak formulations
======================*/

Formulation {
   { Name Wave; Type FemEquation;
      Quantity{
         { Name w; Type Local; NameOfSpace VhNeu;}
         { Name v; Type Local; NameOfSpace VhDir;}
      }

   Equation{
	// Membrane letting free, i.e. Neumann boundary condition on Gama0
      // Taking the initial velocity into acount 
      Galerkin{ DtDof [ ($TimeStep < 2 ? 1 : 0) * Dof{w}, {w} ];
         In Omega; Jacobian JVol; Integration I1;}
         
      Galerkin{ [ -($TimeStep < 2 ? 1 : 0) * InitialVelocity[], {w} ];
         In Omega; Jacobian JVol; Integration I1;}
   
      // Weak formulation after the initialisation of the two first steps
      Galerkin{ DtDtDof [ ($TimeStep < 2 ? 0 : 1) * Dof{w}, {w} ];
         In Omega; Jacobian JVol; Integration I1;}
       
      Galerkin{ [ ($TimeStep < 2 ? 0 : 1) * Dof{Grad w}, {Grad w} ];
         In Omega; Jacobian JVol; Integration I1;}

	// Membrane fixed, i.e. Dirichlet boundary condition on Gama0
      // Taking the initial velocity into acount 
      Galerkin{ DtDof [ ($TimeStep < 2 ? 1 : 0) * Dof{v}, {v} ];
         In Omega; Jacobian JVol; Integration I1;}
         
      Galerkin{ [ -($TimeStep < 2 ? 1 : 0) * InitialVelocity[], {v} ];
         In Omega; Jacobian JVol; Integration I1;}
   
      // Weak formulation after the initialisation of the two first steps
      Galerkin{ DtDtDof [ ($TimeStep < 2 ? 0 : 1) * Dof{v}, {v} ];
         In Omega; Jacobian JVol; Integration I1;}
       
      Galerkin{ [ ($TimeStep < 2 ? 0 : 1) * Dof{Grad v}, {Grad v} ];
         In Omega; Jacobian JVol; Integration I1;}
         
    }
  }
}

/*======================
  Resolution
======================*/

Resolution {
   { Name WaveSolver;
      System{
         { Name SysWave; NameOfFormulation Wave;}
      }
      Operation{
            // Initialisation
            InitSolution[SysWave];
            // Initialisation of the velocity
            TimeLoopTheta[t0, t0 + dt, dt, 0.5 ] {
            Generate[SysWave]; Solve[SysWave];
         }  
            // Computation on [t0 + dt,T]
            TimeLoopNewmark[ t0 + dt, T, dt, beta, gamma ] {
            Generate[SysWave]; Solve[SysWave];
         }
      }
   }
}

/*==============
Post Processing
==============*/

PostProcessing {
   { Name WaveSolver; NameOfFormulation Wave; NameOfSystem SysWave;
      Quantity{
         { Name Free; Value {Local{[{w}]; In AllOmega; Jacobian JVol;}}}
         { Name Fixed; Value {Local{[{v}]; In AllOmega; Jacobian JVol;}}}
      }
   }
}

/*=============
Post Operation
=============*/

PostOperation {
   {Name Map_Omega; NameOfPostProcessing WaveSolver;
      Operation{
         Print[Free, OnElementsOf AllOmega, File "WaveFreeOnOmega.pos"];
         Print[Fixed, OnElementsOf AllOmega, File "WaveFixedOnOmega.pos"];
      }
   }
}

